@@ -14,14 +14,10 @@
  * limitations under the License.
  */
 
-use crate::avm::ParticleParameters;
-use crate::IValue;
+use crate::CallServiceClosure;
 
 use std::path::PathBuf;
 
-pub type CallServiceClosure =
-    Box<dyn Fn(ParticleParameters, Vec<IValue>) -> Option<IValue> + 'static>;
-
 /// Describes behaviour of the AVM.
 pub struct AVMConfig {
     /// Path to a AIR interpreter Wasm file.
@@ -37,6 +33,10 @@ pub struct AVMConfig {
     /// AVM uses it to store data obtained after interpreter execution, and load it as a prev_data by particle_id.
     pub particle_data_store: PathBuf,
 
+    /// Path to a directory to store shared directories called Particle File Vault.
+    /// These directories are shared between services called in the span of a same particle execution.
+    pub vault_dir: PathBuf,
+
     /// Mask used to filter logs, for details see `log_utf8_string` in fluence-faas.
     pub logging_mask: i32,
 }
